<div id="widgetvisualization-{{widget.pk}}" rel="{{widget.pk}}" class="widgetvisualizationdialog" title="{{widget.name}} visualization" width="650px">
    <div style="width:600px;white-space:pre-wrap;font-family:monospace;">
    <pre style="width:600px !important;white-space:pre-wrap">
    <table class="datasetTable">
        <tr>
            <th class="tablecellcenter">Score</th>
            <th class="tablecellcenter">Term</th>      
            <th class="tablecellcenter">Canonical</th> 
        </tr>
        {% for term in terms %}
        <tr class="tablerow">
            <td class="tablecellright">{{term.score}}</td>
            <td class="tablecellleft">{{term.cand}}</td>
            <td class="tablecellleft">{{term.lemma}}</td>
        </tr>
        {% endfor %}
    </table>
    </pre>
    </div>
</div>
