<div id="widgetvisualization-{{widget.pk}}" rel="{{widget.pk}}" class="widgetvisualizationdialog" title="{{widget.name}} visualization">
<div style="width:400px;white-space:pre-wrap;font-family:monospace;">
<pre style="width:400px !important;white-space:pre">
<p style="width:160px; float:left"><b>Accuracy:<b></p><p style="width:100px; float:left">{{result.accuracy|floatformat:"6"}}</p>
<p style="width:160px; float:left"><b>Standard deviation:<b></p><p style="width:100px; float:left">{{result.std|floatformat:"6"}}</p>
</pre>
</div>
</div>